@@ -168,13 +168,12 @@ class _DSAPrivateKey(object):
             encoding,
             format,
             encryption_algorithm,
-            self._backend._lib.PEM_write_bio_DSAPrivateKey,
             evp_pkey,
             self._dsa_cdata
         )
 
 
-@utils.register_interface(dsa.DSAPublicKeyWithNumbers)
+@utils.register_interface(dsa.DSAPublicKeyWithSerialization)
 class _DSAPublicKey(object):
     def __init__(self, backend, dsa_cdata):
         self._backend = backend
@@ -225,7 +224,6 @@ class _DSAPublicKey(object):
         return self._backend._public_key_bytes(
             encoding,
             format,
-            None,
             evp_pkey,
             None
         )
